<?php

namespace Drupal\Tests\group_join_link\Functional;

use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\group\PermissionScopeInterface;
use Drupal\Tests\group\Functional\GroupBrowserTestBase;
use Drupal\user\RoleInterface;

/**
 * Tests for views group leave / join links.
 *
 * @group group
 */
class GroupViewLeaveJoinLinkTest extends GroupBrowserTestBase {

  use StringTranslationTrait;

  /**
   * {@inheritdoc}
   */
  public static $modules = [
    'group',
    'group_test_config',
    'group_join_link',
    'views',
    'test_group_join_link',
  ];

  /**
   * The group we will use to test methods on.
   *
   * @var \Drupal\group\Entity\Group
   */
  protected $group;

  /**
   * Outsider role for the current group.
   *
   * @var \Drupal\group\Entity\GroupRoleInterface
   */
  protected $outsiderRole;

  /**
   * Member role for the current group.
   *
   * @var \Drupal\group\Entity\GroupRoleInterface
   */
  protected $memberRole;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();
    $this->group = $this->createGroup(['type' => 'default']);
    $group_type_id = $this->group->getGroupType()->id();
    $this->outsiderRole = $this->createGroupRole([
      'group_type' => $group_type_id,
      'scope' => PermissionScopeInterface::OUTSIDER_ID,
      'global_role' => RoleInterface::AUTHENTICATED_ID,
      'permissions' => ['view group'],
    ]);
    $this->memberRole = $this->createGroupRole([
      'group_type' => $group_type_id,
      'scope' => PermissionScopeInterface::INSIDER_ID,
      'global_role' => RoleInterface::AUTHENTICATED_ID,
      'permissions' => ['view group'],
    ]);
  }

  /**
   * Test link for outsider without join permission.
   */
  public function testLinkForOuitsiderWithoutJoinPermissions() {
    $account = $this->createUser();
    $this->drupalLogin($account);

    $this->outsiderRole->revokePermission('join group')->save();

    $this->drupalGet('/groups-list');
    $this->assertSession()->statusCodeEquals(200);

    $this->assertSession()->linkNotExists('Join');
  }

  /**
   * Test link for outsider with join permission.
   */
  public function testLinkForOuitsiderWithJoinPermissions() {
    $account = $this->createUser();
    $this->drupalLogin($account);

    $this->outsiderRole->grantPermission('join group')->save();

    $this->drupalGet('/groups-list');
    $this->assertSession()->statusCodeEquals(200);

    $this->assertSession()->linkExists('Join');
  }

  /**
   * Test link for member without leave permission.
   */
  public function testLinkForMemberWithoutLeavePermissions() {
    $account = $this->createUser();
    $this->drupalLogin($account);

    $this->group->addMember($account);

    $this->memberRole->revokePermission('leave group')->save();

    $this->drupalGet('/groups-list');
    $this->assertSession()->statusCodeEquals(200);

    $this->assertSession()->linkNotExists('Leave');
  }

  /**
   * Test link for member with leave permission.
   */
  public function testLinkForMemberWithLeavePermissions() {
    $account = $this->createUser();
    $this->drupalLogin($account);

    $this->memberRole->grantPermission('leave group')->save();

    $this->group->addMember($account);

    $this->drupalGet('/groups-list');
    $this->assertSession()->statusCodeEquals(200);

    $this->assertSession()->linkExists('Leave');
  }

}
